@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2014-2020 Greenbone Networks GmbH
+# Copyright (C) 2014-2021 Greenbone Networks GmbH
 #
 # SPDX-License-Identifier: AGPL-3.0-or-later
 #
@@ -64,7 +64,7 @@ RedisCtx = NewType('RedisCtx', redis.Redis)
 
 
 class OpenvasDB:
-    """ Class to connect to redis, to perform queries, and to move
+    """Class to connect to redis, to perform queries, and to move
     from a KB to another."""
 
     _db_address = None
@@ -82,7 +82,7 @@ class OpenvasDB:
     def create_context(
         cls, dbnum: Optional[int] = 0, encoding: Optional[str] = 'latin-1'
     ) -> RedisCtx:
-        """ Connect to redis to the given database or to the default db 0 .
+        """Connect to redis to the given database or to the default db 0 .
 
         Arguments:
             dbnum: The db number to connect to.
@@ -120,7 +120,7 @@ class OpenvasDB:
     def find_database_by_pattern(
         cls, pattern: str, max_database_index: int
     ) -> Tuple[Optional[RedisCtx], Optional[int]]:
-        """ Search a pattern inside all kbs up to max_database_index.
+        """Search a pattern inside all kbs up to max_database_index.
 
         Returns the redis context for the db and its index as a tuple or
         None, None if the db with the pattern couldn't be found.
@@ -134,7 +134,7 @@ class OpenvasDB:
 
     @staticmethod
     def select_database(ctx: RedisCtx, kbindex: str):
-        """ Use an existent redis connection and select a redis kb.
+        """Use an existent redis connection and select a redis kb.
 
         Arguments:
             ctx: Redis context to use.
@@ -154,7 +154,7 @@ class OpenvasDB:
         start: Optional[int] = LIST_FIRST_POS,
         end: Optional[int] = LIST_LAST_POS,
     ) -> Optional[list]:
-        """ Returns the specified elements from `start` to `end` of the
+        """Returns the specified elements from `start` to `end` of the
         list stored as `name`.
 
         Arguments:
@@ -193,11 +193,18 @@ class OpenvasDB:
         pipe.delete(name)
         results, redis_return_code = pipe.execute()
 
-        return results if redis_return_code else []
+        # The results are left-pushed. To preserver the order
+        # the result list must be reversed.
+        if redis_return_code:
+            results.reverse()
+        else:
+            results = []
+
+        return results
 
     @staticmethod
     def get_key_count(ctx: RedisCtx, pattern: Optional[str] = None) -> int:
-        """ Get the number of keys matching with the pattern.
+        """Get the number of keys matching with the pattern.
 
         Arguments:
             ctx: Redis context to use.
@@ -213,7 +220,7 @@ class OpenvasDB:
 
     @staticmethod
     def remove_list_item(ctx: RedisCtx, key: str, value: str):
-        """ Remove item from the key list.
+        """Remove item from the key list.
 
         Arguments:
             ctx: Redis context to use.
@@ -231,9 +238,11 @@ class OpenvasDB:
 
     @staticmethod
     def get_single_item(
-        ctx: RedisCtx, name: str, index: Optional[int] = LIST_FIRST_POS,
+        ctx: RedisCtx,
+        name: str,
+        index: Optional[int] = LIST_FIRST_POS,
     ) -> Optional[str]:
-        """ Get a single KB element.
+        """Get a single KB element.
 
         Arguments:
             ctx: Redis context to use.
@@ -252,8 +261,10 @@ class OpenvasDB:
         return ctx.lindex(name, index)
 
     @staticmethod
-    def add_single_item(ctx: RedisCtx, name: str, values: Iterable):
-        """ Add a single KB element with one or more values.
+    def add_single_list(ctx: RedisCtx, name: str, values: Iterable):
+        """Add a single KB element with one or more values.
+        The values can be repeated. If the key already exists will
+        be removed an completely replaced.
 
         Arguments:
             ctx: Redis context to use.
@@ -261,17 +272,41 @@ class OpenvasDB:
             value: Elements to add to the key.
         """
         if not ctx:
-            raise RequiredArgument('add_list_item', 'ctx')
+            raise RequiredArgument('add_single_list', 'ctx')
         if not name:
-            raise RequiredArgument('add_list_item', 'name')
+            raise RequiredArgument('add_single_list', 'name')
         if not values:
-            raise RequiredArgument('add_list_item', 'value')
+            raise RequiredArgument('add_single_list', 'value')
+
+        pipe = ctx.pipeline()
+        pipe.delete(name)
+        pipe.rpush(name, *values)
+        pipe.execute()
+
+    @staticmethod
+    def add_single_item(ctx: RedisCtx, name: str, values: Iterable):
+        """Add a single KB element with one or more values. Don't add
+        duplicated values during this operation, but if the the same
+        values already exists under the key, this will not be overwritten.
+
+        Arguments:
+            ctx: Redis context to use.
+            name: key name of a list.
+            value: Elements to add to the key.
+        """
+        if not ctx:
+            raise RequiredArgument('add_single_item', 'ctx')
+        if not name:
+            raise RequiredArgument('add_single_item', 'name')
+        if not values:
+            raise RequiredArgument('add_single_item', 'value')
 
         ctx.rpush(name, *set(values))
 
     @staticmethod
     def set_single_item(ctx: RedisCtx, name: str, value: Iterable):
-        """ Set (replace) a single KB element.
+        """Set (replace) a single KB element. If the same key exists
+        in the kb, it is completed removed. Values added are unique.
 
         Arguments:
             ctx: Redis context to use.
@@ -292,7 +327,7 @@ class OpenvasDB:
 
     @staticmethod
     def get_pattern(ctx: RedisCtx, pattern: str) -> List:
-        """ Get all items stored under a given pattern.
+        """Get all items stored under a given pattern.
 
         Arguments:
             ctx: Redis context to use.
@@ -319,7 +354,7 @@ class OpenvasDB:
 
     @classmethod
     def get_keys_by_pattern(cls, ctx: RedisCtx, pattern: str) -> List[str]:
-        """ Get all items with index 'index', stored under
+        """Get all items with index 'index', stored under
         a given pattern.
 
         Arguments:
@@ -337,9 +372,10 @@ class OpenvasDB:
 
     @classmethod
     def get_filenames_and_oids(
-        cls, ctx: RedisCtx,
+        cls,
+        ctx: RedisCtx,
     ) -> Iterable[Tuple[str, str]]:
-        """ Get all items with index 'index', stored under
+        """Get all items with index 'index', stored under
         a given pattern.
 
         Arguments:
@@ -374,9 +410,9 @@ class BaseKbDB(BaseDB):
     def _add_single_item(
         self, name: str, values: Iterable, utf8_enc: Optional[bool] = False
     ):
-        ''' Changing the encoding format of an existing redis context
+        """Changing the encoding format of an existing redis context
         is not possible. Therefore a new temporary redis context is
-        created to store key-values encoded with utf-8.'''
+        created to store key-values encoded with utf-8."""
         if utf8_enc:
             ctx = OpenvasDB.create_context(self.index, encoding='utf-8')
             OpenvasDB.add_single_item(ctx, name, values)
@@ -384,7 +420,7 @@ class BaseKbDB(BaseDB):
             OpenvasDB.add_single_item(self.ctx, name, values)
 
     def _set_single_item(self, name: str, value: Iterable):
-        """ Set (replace) a single KB element.
+        """Set (replace) a single KB element.
 
         Arguments:
             name: key name of a list.
@@ -393,15 +429,18 @@ class BaseKbDB(BaseDB):
         OpenvasDB.set_single_item(self.ctx, name, value)
 
     def _get_single_item(self, name: str) -> Optional[str]:
-        """ Get a single KB element.
+        """Get a single KB element.
 
         Arguments:
             name: key name of a list.
         """
         return OpenvasDB.get_single_item(self.ctx, name)
 
-    def _get_list_item(self, name: str,) -> Optional[List]:
-        """ Returns the specified elements from `start` to `end` of the
+    def _get_list_item(
+        self,
+        name: str,
+    ) -> Optional[List]:
+        """Returns the specified elements from `start` to `end` of the
         list stored as `name`.
 
         Arguments:
@@ -411,8 +450,11 @@ class BaseKbDB(BaseDB):
         """
         return OpenvasDB.get_list_item(self.ctx, name)
 
+    def _pop_list_items(self, name: str) -> List:
+        return OpenvasDB.pop_list_items(self.ctx, name)
+
     def _remove_list_item(self, key: str, value: str):
-        """ Remove item from the key list.
+        """Remove item from the key list.
 
         Arguments:
             key: key name of a list.
@@ -421,11 +463,11 @@ class BaseKbDB(BaseDB):
         OpenvasDB.remove_list_item(self.ctx, key, value)
 
     def get_result(self) -> Optional[str]:
-        """ Get and remove the oldest result from the list.
+        """Get and remove the oldest result from the list.
 
         Return the oldest scan results
         """
-        return OpenvasDB.pop_list_items(self.ctx, "internal/results")
+        return self._pop_list_items("internal/results")
 
     def get_status(self, openvas_scan_id: str) -> Optional[str]:
         """ Return the status of the host scan """
@@ -439,7 +481,7 @@ class ScanDB(BaseKbDB):
     """ Database for a scanning a single host """
 
     def select(self, kbindex: int) -> "ScanDB":
-        """ Select a redis kb.
+        """Select a redis kb.
 
         Arguments:
             kbindex: The new kb to select
@@ -448,46 +490,10 @@ class ScanDB(BaseKbDB):
         self.index = kbindex
         return self
 
-    def get_scan_id(self):
-        return self._get_single_item('internal/scan_id')
-
-    def get_scan_status(self) -> Optional[str]:
-        """ Get and remove the oldest host scan status from the list.
-
-        Return a string which represents the host scan status.
-        """
-        return OpenvasDB.get_last_list_item(self.ctx, "internal/status")
-
-    def get_host_ip(self) -> Optional[str]:
-        """ Get the ip of host_kb.
-
-        Return a string with the ip of the host being scanned.
-        """
-        return self._get_single_item("internal/ip")
-
-    def get_host_scan_start_time(self) -> Optional[str]:
-        """ Get the timestamp of the scan start from redis.
-
-        Return a string with the timestamp of the scan start.
-        """
-        return OpenvasDB.get_last_list_item(self.ctx, "internal/start_time")
-
-    def get_host_scan_end_time(self) -> Optional[str]:
-        """ Get the timestamp of the scan end from redis.
-
-        Return a string with the timestamp of scan end .
-        """
-        return OpenvasDB.get_last_list_item(self.ctx, "internal/end_time")
-
-    def host_is_finished(self, openvas_scan_id: str) -> bool:
-        """ Returns true if the scan of the host is finished """
-        status = self.get_status(openvas_scan_id)
-        return status == 'finished'
-
 
 class KbDB(BaseKbDB):
     def get_scan_databases(self) -> Iterator[ScanDB]:
-        """ Returns an iterator yielding corresponding ScanDBs
+        """Returns an iterator yielding corresponding ScanDBs
 
         The returned Iterator can't be converted to an Iterable like a List.
         Each yielded ScanDB must be used independently in a for loop. If the
@@ -502,12 +508,9 @@ class KbDB(BaseKbDB):
 
             yield scan_db.select(kbindex)
 
-    def add_scan_id(self, scan_id: str, openvas_scan_id: str):
-        self._add_single_item('internal/{}'.format(openvas_scan_id), ['new'])
-        self._add_single_item(
-            'internal/{}/globalscanid'.format(scan_id), [openvas_scan_id]
-        )
-        self._add_single_item('internal/scanid', [openvas_scan_id])
+    def add_scan_id(self, scan_id: str):
+        self._add_single_item('internal/{}'.format(scan_id), ['new'])
+        self._add_single_item('internal/scanid', [scan_id])
 
     def add_scan_preferences(self, openvas_scan_id: str, preferences: Iterable):
         self._add_single_item(
@@ -517,8 +520,8 @@ class KbDB(BaseKbDB):
     def add_credentials_to_scan_preferences(
         self, openvas_scan_id: str, preferences: Iterable
     ):
-        ''' Force the usage of the utf-8 encoding, since some credentials
-        contain special chars not supported by latin-1 encoding. '''
+        """Force the usage of the utf-8 encoding, since some credentials
+        contain special chars not supported by latin-1 encoding."""
         self._add_single_item(
             'internal/{}/scanprefs'.format(openvas_scan_id),
             preferences,
@@ -537,10 +540,14 @@ class KbDB(BaseKbDB):
     def target_is_finished(self, scan_id: str) -> bool:
         """ Check if a target has finished. """
 
-        openvas_scan_id = self._get_single_item(
-            'internal/{}/globalscanid'.format(scan_id)
-        )
-        status = self._get_single_item('internal/{}'.format(openvas_scan_id))
+        status = self._get_single_item('internal/{}'.format(scan_id))
+
+        if status is None:
+            logger.error(
+                "%s: Target set as finished because redis returned None as "
+                "scanner status.",
+                scan_id,
+            )
 
         return status == 'finished' or status is None
 
@@ -549,12 +556,18 @@ class KbDB(BaseKbDB):
             'internal/{}'.format(openvas_scan_id), ['stop_all']
         )
 
-    def scan_is_stopped(self, openvas_scan_id: str) -> bool:
-        """ Check if the scan should be stopped
-        """
-        status = self._get_single_item('internal/%s' % openvas_scan_id)
+    def scan_is_stopped(self, scan_id: str) -> bool:
+        """Check if the scan should be stopped"""
+        status = self._get_single_item('internal/%s' % scan_id)
         return status == 'stop_all'
 
+    def get_scan_status(self) -> List:
+        """Get and remove the oldest host scan status from the list.
+
+        Return a string which represents the host scan status.
+        """
+        return self._pop_list_items("internal/status")
+
 
 class MainDB(BaseDB):
     """ Main Database """
@@ -568,8 +581,7 @@ class MainDB(BaseDB):
 
     @property
     def max_database_index(self):
-        """Set the number of databases have been configured into kbr struct.
-        """
+        """Set the number of databases have been configured into kbr struct."""
         if self._max_dbindex is None:
             resp = self.ctx.config_get('databases')
 
@@ -578,12 +590,12 @@ class MainDB(BaseDB):
             else:
                 raise OspdOpenvasError(
                     'Redis Error: Not possible to get max_dbindex.'
-                )
+                ) from None
 
         return self._max_dbindex
 
     def try_database(self, index: int) -> bool:
-        """ Check if a redis db is already in use. If not, set it
+        """Check if a redis db is already in use. If not, set it
         as in use and return.
 
         Arguments:
@@ -600,13 +612,12 @@ class MainDB(BaseDB):
         except:
             raise OspdOpenvasError(
                 'Redis Error: Not possible to set %s.' % DBINDEX_NAME
-            )
+            ) from None
 
         return resp == 1
 
     def get_new_kb_database(self) -> Optional[KbDB]:
-        """ Return a new kb db to an empty kb.
-        """
+        """Return a new kb db to an empty kb."""
         for index in range(1, self.max_database_index):
             if self.try_database(index):
                 kbdb = KbDB(index)
@@ -618,17 +629,13 @@ class MainDB(BaseDB):
     def find_kb_database_by_scan_id(
         self, scan_id: str
     ) -> Tuple[Optional[str], Optional["KbDB"]]:
-        """ Find a kb db by via a global scan id
-        """
+        """Find a kb db by via a scan id"""
         for index in range(1, self.max_database_index):
             ctx = OpenvasDB.create_context(index)
-            openvas_scan_id = OpenvasDB.get_single_item(
-                ctx, 'internal/{}/globalscanid'.format(scan_id)
-            )
-            if openvas_scan_id:
-                return (openvas_scan_id, KbDB(index, ctx))
+            if OpenvasDB.get_key_count(ctx, 'internal/{}'.format(scan_id)):
+                return KbDB(index, ctx)
 
-        return (None, None)
+        return None
 
     def release_database(self, database: BaseDB):
         self.release_database_by_index(database.index)
