@@ -52,7 +52,7 @@ BOREAS_SETTING_NAME = "test_alive_hosts_only"
 
 
 class AliveTest(IntEnum):
-    """ Alive Tests. """
+    """Alive Tests."""
 
     ALIVE_TEST_SCAN_CONFIG_DEFAULT = 0
     ALIVE_TEST_TCP_ACK_SERVICE = 1
@@ -114,7 +114,7 @@ class PreferenceHandler:
 
     @property
     def target_options(self) -> Dict:
-        """ Return target options from Scan collection """
+        """Return target options from Scan collection"""
         if self._target_options is not None:
             return self._target_options
 
@@ -156,7 +156,7 @@ class PreferenceHandler:
         return vts_list
 
     def _get_vt_param_type(self, vt: Dict, vt_param_id: str) -> Optional[str]:
-        """ Return the type of the vt parameter from the vts dictionary. """
+        """Return the type of the vt parameter from the vts dictionary."""
 
         vt_params_list = vt.get("vt_params")
         if vt_params_list.get(vt_param_id):
@@ -164,7 +164,7 @@ class PreferenceHandler:
         return None
 
     def _get_vt_param_name(self, vt: Dict, vt_param_id: str) -> Optional[str]:
-        """ Return the type of the vt parameter from the vts dictionary. """
+        """Return the type of the vt parameter from the vts dictionary."""
 
         vt_params_list = vt.get("vt_params")
         if vt_params_list.get(vt_param_id):
@@ -210,7 +210,7 @@ class PreferenceHandler:
         self,
         vts: Dict[str, Dict[str, str]],
     ) -> Tuple[List[str], Dict[str, str]]:
-        """ Add single VTs and their parameters. """
+        """Add single VTs and their parameters."""
         vts_list = []
         vts_params = {}
         vtgroups = vts.pop('vt_groups')
@@ -235,7 +235,7 @@ class PreferenceHandler:
                 param_type = self._get_vt_param_type(vt, vt_param_id)
                 param_name = self._get_vt_param_name(vt, vt_param_id)
 
-                if not param_type or not param_name:
+                if vt_param_id > '0' and (not param_type or not param_name):
                     logger.debug(
                         'Missing type or name for VT parameter %s of %s. '
                         'This VT parameter will not be set.',
@@ -263,11 +263,14 @@ class PreferenceHandler:
                 if type_aux == 'checkbox':
                     vt_param_value = _from_bool_to_str(int(vt_param_value))
 
-                vts_params[
-                    "{0}:{1}:{2}:{3}".format(
-                        vtid, vt_param_id, param_type, param_name
-                    )
-                ] = str(vt_param_value)
+                if vt_param_id == '0':
+                    vts_params["timeout.{0}".format(vtid)] = str(vt_param_value)
+                else:
+                    vts_params[
+                        "{0}:{1}:{2}:{3}".format(
+                            vtid, vt_param_id, param_type, param_name
+                        )
+                    ] = str(vt_param_value)
 
         return vts_list, vts_params
 
@@ -409,16 +412,10 @@ class PreferenceHandler:
                 + 'Mark unrechable Hosts as dead (not scanning)'
             ] = value
 
-            # Also select a method, otherwise Ping Host logs a warning.
-            if alive_test == AliveTest.ALIVE_TEST_CONSIDER_ALIVE:
-                target_opt_prefs_list[
-                    OID_PING_HOST + ':1:checkbox:' + 'Do a TCP ping'
-                ] = 'yes'
-
         return target_opt_prefs_list
 
     def prepare_alive_test_option_for_openvas(self):
-        """ Set alive test option. Overwrite the scan config settings."""
+        """Set alive test option. Overwrite the scan config settings."""
         settings = Openvas.get_settings()
         if settings and (
             self.target_options.get('alive_test')
@@ -499,7 +496,7 @@ class PreferenceHandler:
             self.kbdb.add_scan_preferences(self.scan_id, [pref])
 
     def prepare_reverse_lookup_opt_for_openvas(self):
-        """ Set reverse lookup options in the kb"""
+        """Set reverse lookup options in the kb"""
         if self.target_options:
             items = []
             _rev_lookup_only = int(
@@ -593,7 +590,23 @@ class PreferenceHandler:
 
             if service == 'ssh':
                 port = cred_params.get('port', '')
+                priv_username = cred_params.get('priv_username', '')
+                priv_password = cred_params.get('priv_password', '')
                 cred_prefs_list.append('auth_port_ssh|||' + '{0}'.format(port))
+                cred_prefs_list.append(
+                    OID_SSH_AUTH
+                    + ':7:'
+                    + 'entry:SSH privilege login name:|||{0}'.format(
+                        priv_username
+                    )
+                )
+                cred_prefs_list.append(
+                    OID_SSH_AUTH
+                    + ':8:'
+                    + 'password:SSH privilege password:|||{0}'.format(
+                        priv_password
+                    )
+                )
                 cred_prefs_list.append(
                     OID_SSH_AUTH
                     + ':1:'
